# Generated by Django 3.0.7 on 2020-06-27 09:23

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('app', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='biginteger',
            name='integer',
        ),
    ]
